@@ -3,12 +3,13 @@
  * Copyright (C) 1999,2000,2001,2002 University of Tromso. All rights reserved.
  * Copyright (C) 2002 Invenia Innovation AS. All rights reserved.
  *
- * Author: Feike W. Dillema, feico@pasta.cs.uit.no.
+ * Author: Feike W. Dillema, feico@dillema.net.
  *         based on newbie code by Yusuke DOI, Keio Univ. Murai Lab.
+ * Modified by: Gabor Lencse, see details below
  ****************************************************************************/
 
 /*
- * <$Id: ne_mesg.c,v 3.49 2005/07/04 09:09:22 dillema Exp $>
+ * <$Id: ne_mesg.c,v 3.50 2008/01/22 14:05:09 feico Exp $>
  */
 
 #include "totd.h"
@@ -18,20 +19,63 @@ static int dname_copy (u_char *, u_char *, int);
 static u_char *dname_redirect (u_char *, u_char *);
 static u_char *mesg_read_sec (G_List *, u_char *, int, u_char *, int);
 
-uint16_t mesg_id (void) {
-	static uint16_t id = 0;
+/* Modifications made by Gabor Lencse, lencse@sze.hu for adding random Transaction IDs */
+/* For more details, see our paper (currently under review): */
+/* G. Lencse and S. Repas, "Improving the Performance and Security of the TOTD DNS64 Implementation" */
+/* http://www.hit.bme.hu/~lencse/publications/Improving-TOTD-for-review.pdf */ 
 
-	if (!id) {
-		srandom (time (NULL));
-		id = random ();
-	}
-	id++;
+#define ARRAY_SIZE 0x8000	/* Size of the static array storing permutations */
+#define NUM_USE 0x4000		/* Number of elements used up from the array of permutations */
+#define LOW_START 0x0000	/* Starting value of the lower range */
+#define HIGH_START 0x8000	/* Starting value of the higher range */
 
-	if (T.debug > 4)
-		syslog (LOG_DEBUG, "mesg_id() = %d", id);
-	return id;
+static uint16_t permutation[ARRAY_SIZE]; /* The static array for storing random permutations */
+
+/* Prepare a random permutation of the integers [start, start + ARRAY_SIZE - 1] into the static array */
+/* Algorithm: http://en.wikipedia.org/wiki/Fisher%E2%80%93Yates_shuffle#The_.22inside-out.22_algorithm */
+void make_random_permutation(int start) {
+        int i, j;
+
+        permutation[0] = start;	
+        for (i = 1; i < ARRAY_SIZE; i++) {
+                j = random() * (double)(i + 1) / RAND_MAX; /* random number from the range [0, i] */
+                if (j != i) { 
+                  permutation[i] = permutation[j];
+                }
+                permutation[j] = start + i;
+        }
 }
 
+/* Provide hard to predict unique random DNS Transaction IDs */
+/* by using random permutations and alternating ranges */
+uint16_t mesg_id (void) {
+        static int range = 0; /* indicates that no permutation has been generated yet */
+        static int index;
+
+        if (!range) {
+                srandom(time(0)); 	/* initialize random number generator seed */
+                range = 1;		/* choose the lower range first */
+                make_random_permutation(LOW_START);
+                index = 0;		/* start from the first element */
+        }
+        if ( index == NUM_USE ) {	/* if the pool is exhasuted */
+                if (range == 1) {
+                        range = 2;	/* choose the higher range */
+                        make_random_permutation(HIGH_START);
+                }
+                else {
+                        range = 1;	/* choose the lower range */
+                        make_random_permutation(LOW_START);
+                }
+                index = 0;
+        }
+	if (T.debug > 4) { 
+		syslog (LOG_DEBUG, "mesg_id() = %d", permutation[index]);
+        }
+        return permutation[index++];
+}
+/* End of modifications by Gabor Lencse */
+
 int mesg_make_query (u_char *qname, uint16_t qtype, uint16_t qclass,
 		     uint32_t id, int rd, u_char *buf, int buflen) {
 	char *fn = "mesg_make_query()";
@@ -39,10 +83,10 @@ int mesg_make_query (u_char *qname, uint16_t qtype, uint16_t qclass,
 	int i, written_len;
 	Mesg_Hdr *hdr;
 
-	if (T.debug > 4)
+	if (T.debug > 4) { 
 		syslog (LOG_DEBUG, "%s: (qtype: %s, id: %d): start", fn,
 			string_rtype (qtype), id);
-
+        }
 	hdr = (Mesg_Hdr *) buf;
 
 	/* write header */
@@ -145,9 +189,10 @@ int mesg_dname_cmp (u_char *msg, u_char *dname_mesg, u_char *dname) {
 			return -1;
 
 		if (*dname == EDNS0_ELT_BITLABEL) {
-			if (memcmp (dname_mesg + 1, dname + 1,  len))
+			if (memcmp (dname_mesg + 1, dname + 1,  (size_t)len))
 				return -1;
-		} else if (strncasecmp (dname_mesg + 1, dname + 1, len))
+		} else if (strncasecmp ((const char *)dname_mesg + 1, 
+                                        (const char *)dname + 1, (size_t)len))
 				return -1;
 
 		dname += len + 1;
@@ -166,7 +211,6 @@ int mesg_write_rrset_list (G_List *rrls, u_char *msg, u_char *msg_tail,
 			   uint16_t *cnt) {
 	char *fn = "mesg_write_rrset_list()";
 	u_char *wp_start, *wp_period;
-	Mesg_Hdr *hdr;
 	uint16_t us;
 	uint32_t ul;
 	RRset *rrsp;
@@ -180,7 +224,6 @@ int mesg_write_rrset_list (G_List *rrls, u_char *msg, u_char *msg_tail,
 		return 0;
 
 	wp_start = *wp;
-	hdr = (Mesg_Hdr *) msg;
 
 	for (rrls = rrls->next; rrls->list_data; rrls = rrls->next) {
 		if (T.debug > 4)
@@ -227,12 +270,12 @@ int mesg_write_rrset_list (G_List *rrls, u_char *msg, u_char *msg_tail,
 
 			if (T.debug > 4)
 				syslog (LOG_DEBUG, "%s: now counter = %zd",
-					fn, us);
+					fn, (size_t)us);
 		}
 	}
 
 	if (T.debug > 4)
-		syslog (LOG_DEBUG, "%s: return %d", fn, *wp - wp_start);
+		syslog (LOG_DEBUG, "%s: return %d", fn, (int)(*wp - wp_start));
 
 	return (*wp - wp_start);
 }
@@ -505,7 +548,7 @@ u_char *dname_decompress (u_char *buf, int buflen, u_char *dname,
 
 		if (written_len + token_len >= buflen)
 			return NULL; /* buffer overrun */
-		if (cp + token_len > m_tail)
+		if (m_tail && cp + token_len > m_tail)
 			return NULL; /* out of bounds */
 
 		if (written) {
@@ -529,7 +572,7 @@ u_char *dname_decompress (u_char *buf, int buflen, u_char *dname,
 					return NULL; /* buffer overrun */
 
 				wp = buf;
-				wp += sprintf(wp, "\\[x");
+				wp += sprintf((char *)wp, "\\[x");
 				for (i = 1; i < token_len-1; i++) {
 					u_char d1, d2;
 					uint b;
@@ -537,10 +580,10 @@ u_char *dname_decompress (u_char *buf, int buflen, u_char *dname,
 					b = (int) *(cp + 1 + i);
 					d1 = hex[(b >> 4) & 0x0f];
 					d2 = hex[b & 0x0f];
-					wp += sprintf(wp, "%c%c", d1, d2);
+					wp += sprintf((char *)wp, "%c%c", d1, d2);
 				}
 				bitlength = *(cp + 1) ? *(cp + 1) : 256;
-				wp += sprintf(wp, "/%u].", bitlength);
+				wp += sprintf((char *)wp, "/%u].", bitlength);
 
 				written_len += (wp - buf);
 				buf += written_len;
@@ -634,11 +677,14 @@ char *string_rtype (uint16_t rtype) {
 }
 
 static int dname_copy (u_char *from, u_char *to, int tolen) {
-	int skip, written_len;
+	int skip, llen, written_len;
 
 	written_len = 0;
 	while (*from) {
-		skip = labellen(from) + 1;
+		llen = labellen(from);
+		if (llen == -1) 
+			return -1;
+		skip = llen + 1;
 		written_len += skip;
 		if (written_len >= tolen)
 			return -1;
@@ -723,7 +769,9 @@ static int write_dname (u_char *msg, u_char *msg_tail, uint16_t *dnames,
 
 					/* case-insensitive comparison */
 					if (*cpm != EDNS0_ELT_BITLABEL &&
-					    strncasecmp (cpm+1, cpd+1, *cpm))
+					    strncasecmp ((const char *)cpm+1, 
+                                                         (const char *)cpd+1, 
+                                                         *cpm))
 						break;
 					
 					/* a label matched, move to next one */
