@@ -3,12 +3,12 @@
  * Copyright (C) 1999,2000,2001,2002 University of Tromso. All rights reserved.
  * Copyright (C) 2002 Invenia Innovation AS. All rights reserved.
  *
- * Author: Feike W. Dillema, feico@pasta.cs.uit.no.
+ * Author: Feike W. Dillema, feico@dillema.net.
  *         based on newbie code by Yusuke DOI, Keio Univ. Murai Lab.
  ****************************************************************************/
 
 /*
- * <$Id: request.c,v 3.99 2005/07/04 09:09:22 dillema Exp $>
+ * <$Id: request.c,v 3.100 2008/01/22 14:05:09 feico Exp $>
  */
 
 #include "totd.h"
@@ -70,7 +70,7 @@ int request_start (Context *cont, int tcp) {
         if (*qname == EDNS0_ELT_BITLABEL) {
 		/* We will and may not use qname as string further */
 		qname[0] = '\0';
-	} else if (len != strlen(qname) + 1) {
+	} else if (len != strlen((const char *)qname) + 1) {
 		/* We will and may not use qname as string further */
 		qname[0] = '\0';
 	}
@@ -145,7 +145,7 @@ int request_start (Context *cont, int tcp) {
 
        	syslog (LOG_INFO, "Selected forwarder: %s",
 		sprint_inet((struct sockaddr *)
-			(cont->current_ns->list_data), str));
+			(cont->current_ns->list_data), (char *)str));
 
 	/* forward the actual (modified) query ww want to make */
 	return (do_forward (cont, qname, qclass, qtype, tcp));
@@ -172,7 +172,7 @@ int do_forward(struct context *parent, u_char *qname, uint16_t qclass,
 	cont->timeout = SEARCH_REMOTE_TIMEOUT;
 	cont->current_ns = cont->parent->current_ns;
 
-	strlcpy (cont->qname, qname, MAX_DNAME);
+	strlcpy ((char *)cont->qname, (const char *)qname, MAX_DNAME);
 	cont->q_class = qclass;
 	cont->q_type = qtype;
 	cont->q_id = mesg_id ();
@@ -242,7 +242,7 @@ int request_retry (Context *cont) {
 	else
 		cont->current_ns = cont->current_ns->next;
 
-	dname_decompress (astr, MAX_DNAME, cont->qname, 0, 0, 0);
+	dname_decompress ((u_char *)astr, MAX_DNAME, cont->qname, 0, 0, 0);
        	syslog (LOG_INFO, "Retrying query for %s to nameserver: %s", astr,
 		sprint_inet((struct sockaddr *) (cont->current_ns->list_data),
 			    bstr));
@@ -315,7 +315,7 @@ int recursive_process (Context *cont) {
 	/* initialize vars for additional request we may make */
 	qclass = cont->q_class;
 	qtype = cont->q_type;
-	strlcpy(qname, cont->qname, MAX_DNAME);
+	strlcpy((char *)qname, (const char *)cont->qname, MAX_DNAME);
 
 	syslog (LOG_DEBUG, "%s: work state: %d", fn, cont->work_state);
 
@@ -410,7 +410,7 @@ int recursive_process (Context *cont) {
 				syslog (LOG_DEBUG, "Changed target to A");
 
 				/* same query, but IPv4 address type */
-				strlcpy(qname, cont->qname, MAX_DNAME);
+				strlcpy((char *)qname, (char *)cont->qname, MAX_DNAME);
 				qtype = RT_A;
 
 				cont->work_state = TRICK_REQUEST;
